#!/usr/bin/env python

import argparse
import math
import multiprocessing
import os

from parser import parse_gpx

import draw
from draw.speedometer import speedometer


FPS = 30.0
STILLS_PATH = './stills'


def create_progress_bar(current_frame, last_frame, bar_size=10):
    percent = current_frame / last_frame
    fill_bars = math.floor(bar_size * percent)
    return ("█" * fill_bars) + ("░" * (bar_size - fill_bars))


def create_args_parser():
    parser = argparse.ArgumentParser(description="Creates frames based off a GPX file for overlays")
    parser.add_argument('filename', type=str, help="GPX file to parse")
    parser.add_argument('stills', type=str, help="The folder where to store the still images")
    parser.add_argument('--fps', dest='fps', default=30, help="Frames per second to generate")

    return parser


def draw_frame(data):
    """
    For multiprocessing purposes, this takes a dict
    with the options and the data point and creates
    a frame
    """
    elem = data["datapoint"]
    max_speed = data["max"]["speed"]
    stills_path = data["stills"]
    frame_no = data["frame_no"]
    ctx = draw.initialize(1920, 1080)
    draw.set_background(ctx, 0, 0, 0, 0.0)
    speedometer(ctx, 200, 900, 150, elem[0], max_speed)
    draw.save_to_file(ctx, os.path.join(stills_path, f"still-{frame_no:08}.png"))

    return True


def main(filename, fps, stills_path):
    data = parse_gpx(filename, fps)
    max_speed = data.get_maximum('speed')
    max_elevation = data.get_maximum('elevation')

    print(f"Replaying at {fps} FPS")
    print(f"Saving to {stills_path}")

    if not os.path.exists(stills_path):
        os.makedirs(stills_path)

    frame = 0
    last_frame = data.length()

    batch = []

    print("Preparing batch processing")
    for elem in data.get_datapoints():
        batch.append({"frame_no": frame,
                      "datapoint": elem,
                      "stills": stills_path,
                      "max": {"speed": max_speed}})
        frame += 1

    with multiprocessing.Pool() as p:
        it = p.imap_unordered(draw_frame, batch)
        count = 0

        for elem in it:
            count += 1
            progress_bar = create_progress_bar(count, last_frame, 60)
            print(f"Current Frame: {count} {progress_bar} {(count/last_frame) * 100.0:.2f} %", end="\r")


if __name__ == "__main__":
    parser = create_args_parser()
    options = parser.parse_args()
    main(options.filename, options.fps, options.stills)
